package com.cours.dao;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.cours.exception.GestionPersonnesException;
import com.cours.utils.Constants;
import com.cours.entities.Personne;

public class PersonneDao implements IPersonneDao {

	private static final Log log = LogFactory.getLog(PersonneDao.class);
	private final String persistenceUnit = "PersonnesPU";
	private EntityManagerFactory emf = null;
	private EntityManager em = null;
	private EntityTransaction transaction = null;

	@Override
	public List<Personne> findAll() {
		List<Personne> persons = null;
		try {
			emf = Persistence.createEntityManagerFactory(persistenceUnit);
			em = emf.createEntityManager();
			// persons = em.createQuery("select person from Personne person",
			// Personne.class).getResultList();
			persons = em.createNamedQuery("Personne.findAll", Personne.class).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException("Une erreur s'est produite lors de la recupération des personnes.", 1);
		} finally {
			JPAHelper.closeEntityManagerResources(emf, em);
		}
		log.debug("persons : " + persons);
		return persons;
	}

	@Override
	public Personne create(Personne person) {
		if (person != null) {
			try {
				emf = Persistence.createEntityManagerFactory(persistenceUnit);
				em = emf.createEntityManager();
				transaction = em.getTransaction();
				transaction.begin();
				em.persist(person);
				transaction.commit();
			} catch (Exception e) {
				log.error("Erreur lors de l'execution de la methode, Exception : " + e);
				if (transaction != null && transaction.isActive()) {
					transaction.rollback();
				}
				throw new GestionPersonnesException(
						"Une erreur s'est produite lors de la création de la personne : [id = " + person.getIdPersonne()
								+ " , prenom = " + person.getPrenom() + " , nom = " + person.getNom() + "]",
						2);
			} finally {
				JPAHelper.closeEntityManagerResources(emf, em);
			}
		}
		return person;
	}

	@Override
	public Personne update(Personne person) {
		if (person != null) {
			try {
				emf = Persistence.createEntityManagerFactory(persistenceUnit);
				em = emf.createEntityManager();
				transaction = em.getTransaction();
				transaction.begin();
				person = em.merge(person);
				transaction.commit();
			} catch (Exception e) {
				log.error("Erreur lors de l'execution de la methode, Exception : " + e);
				if (transaction != null && transaction.isActive()) {
					transaction.rollback();
				}
				throw new GestionPersonnesException(
						"Une erreur s'est produite lors de la mise à jour de la personne : [id = "
								+ person.getIdPersonne() + " , prenom = " + person.getPrenom() + " , nom = "
								+ person.getNom() + "]",
						3);
			} finally {
				JPAHelper.closeEntityManagerResources(emf, em);
			}
		}
		return person;
	}

	@Override
	public boolean delete(Personne person) {
		boolean isOk = false;
		if (person != null) {
			try {
				emf = Persistence.createEntityManagerFactory(persistenceUnit);
				em = emf.createEntityManager();
				transaction = em.getTransaction();
				transaction.begin();
				person = em.find(Personne.class, person.getIdPersonne());
				em.remove(em.merge(person));
				transaction.commit();
				isOk = em.find(Personne.class, person.getIdPersonne()) == null;
			} catch (Exception e) {
				log.error("Erreur lors de l'execution de la methode, Exception : " + e);
				if (transaction != null && transaction.isActive()) {
					transaction.rollback();
				}
				throw new GestionPersonnesException(
						"Une erreur s'est produite lors de la suppression de la personne : [id = "
								+ person.getIdPersonne() + " , prenom = " + person.getPrenom() + " , nom = "
								+ person.getNom() + "]",
						4);
			} finally {
				JPAHelper.closeEntityManagerResources(emf, em);
			}
		}
		return isOk;
	}

	@Override
	public Personne findById(Integer id) {
		Personne person = null;
		try {
			emf = Persistence.createEntityManagerFactory(persistenceUnit);
			em = emf.createEntityManager();
			person = em.find(Personne.class, id);
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException(
					"Une erreur s'est produite lors de la recupération de la personne [id = " + person.getIdPersonne()
							+ "]",
					5);
		} finally {
			JPAHelper.closeEntityManagerResources(emf, em);
		}
		return person;
	}

	@Override
	public List<Personne> findByPrenom(String prenom) {
		List<Personne> personnes = null;
		try {
			emf = Persistence.createEntityManagerFactory(persistenceUnit);
			em = emf.createEntityManager();
			personnes = em
					.createQuery("select person from Personne person where person.prenom=:firstName", Personne.class)
					.setParameter("firstName", prenom).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException(
					"Une erreur s'est produite lors de la recupération des personnes [prenom = " + prenom + "]", 6);
		} finally {
			JPAHelper.closeEntityManagerResources(emf, em);
		}
		return personnes;
	}

	@Override
	public List<Personne> findByNom(String nom) {
		List<Personne> personnes = null;
		try {
			emf = Persistence.createEntityManagerFactory(persistenceUnit);
			em = emf.createEntityManager();
			personnes = em.createQuery("select person from Personne person where person.nom=:lastName", Personne.class)
					.setParameter("lastName", nom).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException(
					"Une erreur s'est produite lors de la recupération des personnes [nom = " + nom + "]", 7);
		} finally {
			JPAHelper.closeEntityManagerResources(emf, em);
		}
		return personnes;
	}

	@Override
	public List<Personne> findByPrenomNom(String prenom, String nom) {
		List<Personne> personnes = null;
		try {
			emf = Persistence.createEntityManagerFactory(persistenceUnit);
			em = emf.createEntityManager();
			personnes = em.createQuery(
					"select person from Personne person where  person.prenom=:firstName and person.nom=:lastName",
					Personne.class).setParameter("firstName", prenom).setParameter("lastName", nom).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException(
					"Une erreur s'est produite lors de la recupération des personnes [nom = " + nom + "]", 8);
		} finally {
			JPAHelper.closeEntityManagerResources(emf, em);
		}
		return personnes;
	}

	@Override
	public List<Personne> findByCodePostal(String codePostal) {
		List<Personne> personnes = null;
		try {
			emf = Persistence.createEntityManagerFactory(persistenceUnit);
			em = emf.createEntityManager();
			personnes = em
					.createQuery("select person from Personne person where person.codePostal=:zipCode", Personne.class)
					.setParameter("zipCode", codePostal).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException(
					"Une erreur s'est produite lors de la recupération des personnes [codePostal = " + codePostal + "]",
					9);
		} finally {
			JPAHelper.closeEntityManagerResources(emf, em);
		}
		return personnes;
	}

	@Override
	public List<Personne> findByVille(String ville) {
		List<Personne> personnes = null;
		try {
			emf = Persistence.createEntityManagerFactory(persistenceUnit);
			em = emf.createEntityManager();
			personnes = em.createQuery("select person from Personne person where person.ville=:town", Personne.class)
					.setParameter("town", ville).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException(
					"Une erreur s'est produite lors de la recupération des personnes [ville = " + ville + "]", 10);
		} finally {
			JPAHelper.closeEntityManagerResources(emf, em);
		}
		return personnes;
	}

	@Override
	public List<Personne> findPersonnesObese() {
		List<Personne> persons = null;
		try {
			emf = Persistence.createEntityManagerFactory(persistenceUnit);
			em = emf.createEntityManager();
			persons = em.createQuery(
					"select person from Personne person where  person.poids/(person.taille/100 * person.taille/100) >:paramObese",
					Personne.class).setParameter("paramObese", Constants.LIMITE_SUP_SURPOIDS).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException("Une erreur s'est produite lors de la recupération des personnes.", 1);
		} finally {
			JPAHelper.closeEntityManagerResources(emf, em);
		}
		log.debug("persons : " + persons);
		return persons;
	}

	@Override
	public List<Personne> findPersonnesMaigre() {
		List<Personne> persons = null;
		try {
			emf = Persistence.createEntityManagerFactory(persistenceUnit);
			em = emf.createEntityManager();
			persons = em.createQuery(
					"select person from Personne person where  person.poids/(person.taille/100 * person.taille/100) <:paramMaigre",
					Personne.class).setParameter("paramMaigre", Constants.LIMITE_INF_MAIGRE).getResultList();
		} catch (Exception e) {
			log.error("Erreur lors de l'execution de la methode, Exception : " + e);
			throw new GestionPersonnesException("Une erreur s'est produite lors de la recupération des personnes.", 1);
		} finally {
			JPAHelper.closeEntityManagerResources(emf, em);
		}
		log.debug("persons : " + persons);
		return persons;
	}

	@Override
	public List<Personne> createBatchPersonnes(List<Personne> persons) {
		boolean isOk = false;
		int nbPersonneCreated = 0;
		log.debug("1 : persons : " + persons);
		if (persons != null && !persons.isEmpty()) {
			emf = Persistence.createEntityManagerFactory(persistenceUnit);
			em = emf.createEntityManager();
			transaction = em.getTransaction();
			transaction.begin();
			for (Personne person : persons) {
				try {
					em.persist(person);
					nbPersonneCreated++;
				} catch (Exception e) {
					log.error("Erreur lors de l'execution de la methode, Exception : " + e);
					if (transaction != null && transaction.isActive()) {
						transaction.rollback();
					}
					throw new GestionPersonnesException(
							"Une erreur s'est produite lors de la création en masse des personnes, Exception pour la personne : [id = "
									+ person.getIdPersonne() + " , prenom = " + person.getPrenom() + " , nom = "
									+ person.getNom() + "]",
							6);
				}
			}
			if (transaction != null && transaction.isActive()) {
				transaction.commit();
				JPAHelper.closeEntityManagerResources(emf, em);
			}
			log.debug("2 : persons : " + persons);
			if (nbPersonneCreated != persons.size()) {
				persons = null;
			}
		}
		log.debug("isOk : " + isOk + " , persons : " + persons);
		return persons;
	}

	@Override
	public Boolean deleteBatchPersonnes(List<Personne> persons) {
		boolean isOk = false;
		int nbPersonneDeleted = 0;
		log.debug("1 : persons : " + persons);
		if (persons != null && !persons.isEmpty()) {
			emf = Persistence.createEntityManagerFactory(persistenceUnit);
			em = emf.createEntityManager();
			transaction = em.getTransaction();
			transaction.begin();
			for (Personne person : persons) {
				try {
					person = em.find(Personne.class, person.getIdPersonne());
					em.remove(em.merge(person));
					nbPersonneDeleted++;
				} catch (Exception e) {
					log.error("Erreur lors de l'execution de la methode, Exception : " + e);
					if (transaction != null && transaction.isActive()) {
						transaction.rollback();
					}
					throw new GestionPersonnesException(
							"Une erreur s'est produite lors de la suppresion en masse des personnes, Exception pour la personne : [id = "
									+ person.getIdPersonne() + " , prenom = " + person.getPrenom() + " , nom = "
									+ person.getNom() + "]",
							7);
				}
			}
			if (transaction != null && transaction.isActive()) {
				transaction.commit();
				JPAHelper.closeEntityManagerResources(emf, em);
			}
			log.debug("2 : personnes : " + persons);
			if (nbPersonneDeleted == persons.size()) {
				isOk = true;
			}
		}
		log.debug("isOk : " + isOk + " , persons : " + persons);
		return isOk;
	}
}
